import dash_bootstrap_components as dbc
from dash import html, dcc
from webapp.styles import *
from webapp.legend import legend_object, legend_object_individual
from webapp.save_modal import save_modal, save_process_modal
from webapp.selected_elements import empty_card, empty_card_individual
import dash_daq as daq

conformance_granularity = dcc.RadioItems(options=[{'label': 'Monthly', 'value': 'monthly'},
                                                  {'label': 'Weekly', 'value': 'weekly'},
                                                  {'label': 'Daily', 'value': 'daily'}],
                                         value='monthly',
                                         id='conformance-granularity',
                                        inputStyle={"margin": "3px"})

def layout_discovery(project_key):
    left_style = tab_button_style_freq_time_active.copy()
    left_style['border-right'] = '1px solid #666666'
    return [dbc.Row([dbc.Col([html.Button('Frequency', id='freq-tab', style=tab_button_style_freq_time_active),
             html.Button('Time', id='time-tab', style=tab_button_style_freq_time)],
            id='radio-buttons-time-freq', style={'float': 'right',
                                       'margin': 0,
                                       'padding': 0}, md=3),
                     dbc.Col(id='graph-legend', 
                             children=legend_object(1, 100),
                             md=6),
                     dbc.Col([html.Button('Save', id='save-process-button', style=button_save_style),
                              html.Button(html.I(className="fas fa-caret-down"), id='save-process-dropdown', style=button_save_style_caret),
                              dbc.Collapse([dbc.Row(html.Button('Save as', id='save-as-button', style=button_save_as_style), 
                              style={'padding': 0, 'margin': 0}),
                                            dbc.Row(html.Button('Export as png', id='save-viz', style=button_save_as_style),
                                style={'padding': 0, 'margin': 0})], id='save-as-collapse', is_open=False,
                                style={'position': 'absolute',
                                       'top': 'auto',
                                       'right': '6px',
                                       'z-index':'10'})],
                                                id='buttons-save-process', style={'text-align': 'right',
                                              'float': 'right',
                                               'margin': 0,
                                               'margin-bottom': '-12px',
                                               'padding': 0,
                                               'padding-right': '18px'}, md=3),
                    save_modal(project_key),
                    save_process_modal(project_key),
                dcc.Location(id='location')], 
                    style={'margin-top': '7px', 'margin-left': '0px', 'margin-bottom': '12px'}),
             dbc.Card(id='selected',
                      className='w-25',
                      children=empty_card(),
                     style={'position': 'absolute',
                            'z-index': '999999',
                            'display': 'None'}),
             dcc.Loading(id='graph-loading', type='circle', 
                         children=html.Div(id='graph-workflow'))]

def layout_conformance():
    return [dbc.Row([dbc.Col(html.P('Overall conformance', 
                                    style={'font-weight': 'bold', 'text-align': 'left'}), md=4),
                     dbc.Col(conformance_granularity, md=4),
                     dbc.Col([html.Button(html.I(className="fas fa-list"), id='conformance-table-tab', style=tab_button_style_list_chart),
             html.Button(html.I(className="fas fa-chart-line"), id='conformance-graph-tab', style=tab_button_style_list_chart_active)],
                   style={'text-align': 'right', 'padding': 0}, 
                             id='list-chart-radio-buttons', md=4)
            ], style={'clear': 'both', 'padding': '12px 22px 12px 0px'}), 
            dcc.Loading(id='graphs', type='circle')]

def layout_individual(project_key):
    left_style = tab_button_style_freq_time_active.copy()
    left_style['border-right'] = '1px solid #666666'
    return [dbc.Row([dbc.Col(html.P('Individual traces', 
                                    style={'font-weight': 'bold', 'text-align': 'left'}), md=4),
                     dbc.Col(md=4),
                     dbc.Col([html.Button(html.I(className="fas fa-list"), id='conformance-table-tab', style=tab_button_style_list_chart_active),
             html.Button(html.I(className="fas fa-chart-line"), id='conformance-graph-tab', style=tab_button_style_list_chart)],
                   style={'text-align': 'right', 'padding': 0}, 
                             id='list-chart-radio-buttons', md=4)
            ], style={'clear': 'both', 'padding': '12px 22px 12px 0px'}),
            dcc.Loading(id='individual-processes', type='circle'),
            dbc.Row([dbc.Col([html.Button('Frequency', id='freq-tab-individual', style=left_style),
             html.Button('Time', id='time-tab-individual', style=tab_button_style_freq_time)],
            id='radio-buttons-time-freq-individual', style={'float': 'right',
                                       'margin': 0,
                                       'padding': 0}, md=3),
                     dbc.Col(id='graph-legend-individual', 
                             children=legend_object_individual(0, 100),
                             md=6),
                    dbc.Col(html.Button('Export', id='save-viz', style=button_style), md=3),
                    save_modal(project_key),
                dcc.Location(id='location')],
                    id='individual-graph-top-bar',
                    style={'margin-top': '7px', 'margin-left': '0px',
                          'display': 'None'}),
            html.Div(id='individual-graph'),
            dbc.Card(id='selected-individual',
                      className='w-25',
                      children=empty_card_individual(),
                     style={'position': 'absolute',
                            'z-index': '999999',
                            'display': 'None'})]
